/* ITUNES */
say "Launching iTunes..."
itunes = .OLEObject~New("iTunes.Application")

Call sysSleep 2


-- counter vars
acount = 1
scount = 1
gcount = 1

-- empty arrays
art = .array~new
songs = .array~new
genres = .array~new

say "Fetching Artists from iTunes Library..."
DO j OVER itunes~LibraryPlaylist~Tracks()
     IF (art~hasItem(j~Artist) = 0) THEN
       DO
        art[acount] = j~Artist
        acount = acount + 1
       END
END

-- new 2-dim array
newart = .array~new

SAY "Fetching Year of Birth from imdb.com via IE for Artist:"
DO j OVER art
  IF j = "Falco" | j = "Halsey" | j = "Billie Eilish" THEN newart[art~index(j)] = (j, FindBirthDate(j))
                 ELSE newart[art~index(j)] = (j, "") 
END


SAY "Fetching Genres from iTunes Library..."
DO j OVER itunes~LibraryPlaylist~Tracks()
    IF (genres~hasItem(j~Genre) = 0) THEN
      DO
        genres[gcount] = j~Genre
        gcount = gcount + 1;
      END
END

SAY "Fetching Tracks from iTunes Library, preparing 4-dim data tuples for export..."
DO j OVER itunes~LibraryPlaylist~Tracks()
    IF (songs~hasItem(j~Name) = 0) THEN
      DO
        songs[scount] = (scount, j~Name, FindArtistIdByName(j~Artist), FindGenreIdByName(j~Genre))
        scount = scount + 1
      END
END

SAY "Exiting iTunes..."
itunes~quit

/* ACCESS */

SAY "Setup Access DB..."
/* Initialize string to database path */
pathDB = "C:\Users\dev\rexx\project\musicDB.mdb"
/* Create new instance of Microsoft Access */
appAccess = .OLEObject~new("Access.Application")
/* Open database in Microsoft Access window */
appAccess~NewCurrentDatabase(pathDB)
/* Get database object variable */
dbs = appAccess~Currentdb


/* Create new tables */
SAY "Setup Table Definitions..."
tdfSongs = dbs~CreateTableDef("Songs")
tdfArtists = dbs~CreateTableDef("Artists")
tdfGenres = dbs~CreateTableDef("Genres")

/* Create Field objects in all tables */

SAY "Setup Field Definitions..."
-- 1. Songs
-- primary key, hence Integer value
fldSongID = tdfSongs~CreateField("SongID", appAccess~getConstant("db_Integer"), 3)
fldTitle = tdfSongs~CreateField("Title", appAccess~getConstant("db_Text"), 128)
-- is a foreign key, therefore integer value
fldForeignArtistID = tdfSongs~CreateField("ArtistID", appAccess~getConstant("db_Integer"), 3)
-- same with genre
fldForeignGenreID = tdfSongs~CreateField("GenreID", appAccess~getConstant("db_Integer"), 3)

-- 2. Artists
-- primary key, hence Integer value
fldArtistID = tdfArtists~CreateField("ArtistID", appAccess~getConstant("db_Integer"), 3)
fldArtistName = tdfArtists~CreateField("ArtistName", appAccess~getConstant("db_Text"), 128)
fldBirthdate = tdfArtists~CreateField("Birthdate", appAccess~getConstant("db_Text"), 4) -- if date: db_Date, 6
--fldForeignSongID = tdfArtists~CreateField("SongID", appAccess~getConstant("db_Integer"), 3)

-- 3. Genres
-- primary key -> Int value
fldGenreID = tdfGenres~CreateField("GenreID", appAccess~getConstant("db_Integer"), 3)
fldGenreName = tdfGenres~CreateField("GenreName", appAccess~getConstant("db_Text"), 40)

/* Append Field objects to Fields collection of the TableDef objects */
SAY "Setup Field Collections..."
-- 1. Songs
tdfSongs~Fields~Append(fldSongID)
tdfSongs~Fields~Append(fldTitle)
tdfSongs~Fields~Append(fldForeignArtistID)
tdfSongs~Fields~Append(fldForeignGenreID)

-- 2. Artists
tdfArtists~Fields~Append(fldArtistID)
tdfArtists~Fields~Append(fldArtistName)
tdfArtists~Fields~Append(fldBirthdate)

-- 3. Genres
tdfGenres~Fields~Append(fldGenreID)
tdfGenres~Fields~Append(fldGenreName)

SAY "Setup Indizes..."
/* Create Index object for Artists table */
idxArt = tdfArtists~CreateIndex("ArtistIDIndex")
/* Create Field object */
fld = idxArt~CreateField("ArtistID")
/* Append Field object to Fields collection of the Index object */
idxArt~Fields~Append(fld)
/* Define the index as unique */
idxArt~Unique = .True
/* Append the Index object to the TableDef object */
tdfArtists~Indexes~Append(idxArt)

/* Create Index object for Genres table */
idxGen = tdfGenres~CreateIndex("GenreIDIndex")
fld = idxgen~CreateField("GenreID")
idxGen~Fields~Append(fld)
idxGen~Unique = .True
tdfGenres~Indexes~Append(idxGen)

/* Append TableDef objects to TableDefs collection */
SAY "Setup TableDef Collection..."
dbs~TableDefs~Append(tdfSongs)
dbs~TableDefs~Append(tdfArtists)
dbs~TableDefs~Append(tdfGenres)

titles = songs
artists = newart
genres = genres

/* Add records to the Songs Recordset */
SAY "Adding Track Records to Recordset..."
rstSongs = dbs~OpenRecordset("Songs")
DO i OVER titles
  rstSongs~AddNew
  rstSongs~SongID = i[1]
  rstSongs~Title = i[2]
  rstSongs~ArtistID = i[3]
  rstSongs~GenreID = i[4]
  rstSongs~Update
END
rstSongs~Close

/* Add records to the Artists Recordset */
SAY "Adding Artist Records to Recordset..."
rstArt = dbs~OpenRecordset("Artists")
DO i OVER artists
  rstArt~AddNew
  rstArt~ArtistID = artists~index(i)
  rstArt~ArtistName = i[1]
  IF i[2]~length > 0 THEN
    rstArt~BirthDate = i[2]
  rstArt~Update
END
rstArt~Close

/* Add records to the Genres RecordSet */
SAY "Adding Genre Records to Recordset..."
rstGen = dbs~OpenRecordSet("Genres")
DO i OVER genres
  rstGen~AddNew
  rstGen~GenreID = genres~index(i)
  rstGen~GenreName = i
  rstGen~Update
END
rstGen~Close

/* Set Relations */
-- Der Primaerschluessel der 1-Beziehung wird als Fremdschluessel in die N-Beziehung eingesetzt.

Call sysSleep 10
-- 1. Artist-Song-Relation
/* Create ArtistsSongs Relation object */
SAY "Setup Artist-Songs Relation..."
newRelation = dbs~CreateRelation("ArtistsSongs", tdfArtists~Name, tdfSongs~Name, appAccess~getConstant("dbRelationUpdateCascade"))
/* Create a Field object of the primary table */
relatingField = newRelation~CreateField("ArtistID")
/* Define the matching field from the related table */
relatingField~ForeignName = "ArtistID"
/* Add the Field to the relation's Fields collection */
newRelation~Fields~Append(relatingField)
/* Add the relation to the database */
dbs~Relations~Append(newRelation)

-- 2. Genre-Song-Relation
SAY "Setup Genre-Song Relation..."
newRelation = dbs~CreateRelation("SongsGenres", tdfGenres~Name, tdfSongs~Name, appAccess~getConstant("dbRelationUpdateCascade"))
relatingField = newRelation~CreateField("GenreID")
relatingField~ForeignName = "GenreID"
newRelation~Fields~Append(relatingField)
dbs~Relations~Append(newRelation)

SAY "All Done!"

FindSongIdByName :
  DO i OVER songs
    IF i == ARG(1) THEN songId = songs~index(i)
  END
  RETURN songId

FindGenreIdByName :
  DO i OVER genres
    IF i == ARG(1) THEN genreId = genres~index(i)
  END
  RETURN genreId

FindArtistIdByName :
  DO i OVER art
    IF i == ARG(1) THEN nameId = art~index(i)
  END
  RETURN nameId

FindBirthDate :
artist = arg(1)
say artist 
IE = .OLEObject~new("InternetExplorer.Application")
IE~Visible =.true
IE~Navigate("https://www.imdb.com/find?q="artist"&ref_=nv_sr_sm")

call SysSleep 3

doc = IE~document
-- body = doc~body
-- textrange = body~CreateTextRange
-- text = textrange~Text
-- html = body~innerHTML

-- el = html~getElementsByTagName("a")(0)
-- el = doc~getElementsByClassName("result_text")[1]
firstHeader = doc~querySelectorAll("h3.findSectionHeader")[0]~innerText
IF firstHeader = "Names" THEN id = 0
                         ELSE id = 1
-- say id
el = doc~querySelectorAll("table.findList")[id]~querySelector("a")
eltext = el~getAttribute("href")
-- say eltext

url = "https://www.imdb.com"eltext
-- say url

IE~Navigate(url)
call sysSleep 3

doc = IE~document
body = doc~body
textrange = body~CreateTextRange
text = textrange~Text

-- parse var text "Names" art .
parse var text . "Born: " Birthdate "in" .
parse var Birthdate m d","y
year = y~strip
IE~Quit
return year
